<?php

namespace Drupal\ipstack;

/**
 * @file
 * Contains \Drupal\ipstack\Ipstack.
 */

use Drupal\Core\Url;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Render\Markup;
use Drupal\Core\Cache\CacheBackendInterface;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class Ipstack.
 *
 * Provides ipstack.com API.
 *
 * @ingroup ipstack
 */
class Ipstack {

  use StringTranslationTrait;

  /**
   * IP address.
   *
   * @var string
   */
  protected $ip;

  /**
   * Ipstack options.
   *
   * @var array
   */
  protected $options;

  /**
   * Is ipstack data from cache?
   *
   * @var bool
   */
  protected $cacheData;

  /**
   * Ipstack constructor.
   *
   * @param string $ip
   *   IP address.
   * @param array $options
   *   Ipstack options.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   String translation object.
   */
  public function __construct($ip = '', array $options = [], TranslationInterface $string_translation = NULL) {
    $this->setIp($ip);
    $this->setOptions($options);
    $this->stringTranslation = $string_translation;
    if (!$this->stringTranslation) {
      $this->stringTranslation = \Drupal::service('string_translation');
    }
    $this->cacheData = FALSE;
  }

  /**
   * Set IP address.
   *
   * @param string $ip
   *   IP address.
   */
  public function setIp($ip) {
    $this->ip = trim($ip);

    // If IP is empty set current user IP.
    if (empty($this->ip)) {
      $this->ip = \Drupal::request()->getClientIp();
    }
  }

  /**
   * Set ipstack options. (config value if empty).
   *
   * @param string $options
   *   Ipstack options.
   */
  public function setOptions($options) {
    if (empty($options['access_key'])) {
      $access_key = \Drupal::config('ipstack.settings')->get('access_key');
      $options['access_key'] = $access_key;
    }
    $this->options = $options;
  }

  /**
   * Get ipstack URL.
   *
   * @return string
   *   Ipstack URL for data retrieving.
   */
  public function getUrl() {
    $prot = !empty(\Drupal::config('ipstack.settings')->get('use_https')) ? 'https' : 'http';
    $url_options = ['absolute' => TRUE, 'query' => $this->options];
    $url = Url::fromUri($prot . "://api.ipstack.com/" . $this->ip, $url_options);
    return $url->toString();
  }

  /**
   * Get ipstack data.
   *
   * @return arrau
   *   Ipstack data responce.
   */
  public function getData() {
    $use_cache = \Drupal::config('ipstack.settings')->get('use_cache');
    $this->cacheData = FALSE;

    if ($use_cache) {
      $cid = 'ipstack:ip_' . $this->ip;
      $cache = \Drupal::cache()->get($cid);
      if (!empty($cache)) {
        $this->cacheData = TRUE;
        return ['data' => $cache->data];
      }
    }

    // Access Key is required.
    if (empty($this->options['access_key'])) {
      return ['error' => $this->t('Empty access_key')];
    }

    // Get data from api.ipstack.com .
    $client = \Drupal::httpClient();
    $responce = $client->get($this->getUrl());
    $status = $responce->getStatusCode();
    if ($status != Response::HTTP_OK) {
      return ['error' => $this->t('Got responce status @status', ['@status' => $status])];
    }

    $data = (string) $responce->getBody();
    if ($use_cache) {
      \Drupal::cache()->set($cid, $data, CacheBackendInterface::CACHE_PERMANENT, ['ipstack']);
    }

    return ['data' => $data];
  }

  /**
   * Show result.
   */
  public function showResult() {
    $url = $this->getUrl();
    $messenger = \Drupal::messenger();
    $msg = $this->t("Request: <a href='@url' target='_new'>@url</a>", ['@url' => $url]);
    $messenger->addMessage($msg);

    $data = $this->getData();

    if (!empty($data['error'])) {
      $msg = $data['error'];
      $messenger->addMessage($msg, 'error');
    }

    if (!empty($data['data'])) {
      $data = $data['data'];
      if ($this->cacheData) {
        $messenger->addMessage($this->t('From cache'));
      }

      // Decode JSON object.
      if (empty($this->options['output']) || $this->options['output'] == 'json') {
        $data = json_decode($data);
      }

      $status = 'status';
      if (!empty($data->error)) {
        $data = $data->error;
        $status = 'error';
      }

      $msg = $this->t('Responce:');
      if (is_array($data) || is_object($data)) {
        $msg .= ' <pre>' . print_r($data, 1) . '</pre>';
        $msg = Markup::create($msg);
      }
      else {
        $msg .= $data;
      }
      $messenger->addMessage($msg, $status);
    }

    $messenger->all();
  }

}
